@@ -1,6 +1,5 @@
-#!/usr/bin/env python2
-#    TerminatorConfig - layered config classes
-#    Copyright (C) 2006-2010  cmsj@tenshu.net
+#  TerminatorConfig - layered config classes
+#  Copyright (C) 2006-2010  cmsj@tenshu.net
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -71,13 +70,13 @@ KeyError: 'ConfigBase::get_item: unknown key algo'
 
 """
 
-import platform
 import os
+import shutil
 from copy import copy
-from configobj.configobj import ConfigObj, flatten_errors
-from configobj.validate import Validator
-from borg import Borg
-from util import dbg, err, DEBUG, get_config_dir, dict_diff
+from configobj import ConfigObj, flatten_errors
+from validate import Validator
+from .borg import Borg
+from .util import dbg, err, DEBUG, get_system_config_dir, get_config_dir, dict_diff
 
 from gi.repository import Gio
 
@@ -103,6 +102,7 @@ DEFAULTS = {
             'use_custom_url_handler': False,
             'custom_url_handler'    : '',
             'disable_real_transparency' : False,
+            'title_at_bottom'       : False,
             'title_hide_sizetext'   : False,
             'title_transmit_fg_color' : '#ffffff',
             'title_transmit_bg_color' : '#c80003',
@@ -119,12 +119,20 @@ DEFAULTS = {
             'title_use_system_font' : True,
             'title_font'            : 'Sans 9',
             'putty_paste_style'     : False,
+            'putty_paste_style_source_clipboard': False,
             'smart_copy'            : True,
+            'clear_select_on_copy'  : False,
+            'line_height'           : 1.0,
+            'case_sensitive'        : True,
+            'invert_search'         : False,
         },
         'keybindings': {
             'zoom_in'          : '<Control>plus',
             'zoom_out'         : '<Control>minus',
             'zoom_normal'      : '<Control>0',
+			'zoom_in_all'	   : '',
+			'zoom_out_all'	   : '',
+			'zoom_normal_all'  : '',
             'new_tab'          : '<Shift><Control>t',
             'cycle_next'       : '<Control>Tab',
             'cycle_prev'       : '<Shift><Control>Tab',
@@ -174,6 +182,7 @@ DEFAULTS = {
             'reset'            : '<Shift><Control>r',
             'reset_clear'      : '<Shift><Control>g',
             'hide_window'      : '<Shift><Control><Alt>a',
+            'create_group'     : '',
             'group_all'        : '<Super>g',
             'group_all_toggle' : '',
             'ungroup_all'      : '<Shift><Super>g',
@@ -182,9 +191,9 @@ DEFAULTS = {
             'ungroup_tab'      : '<Shift><Super>t',
             'new_window'       : '<Shift><Control>i',
             'new_terminator'   : '<Super>i',
-            'broadcast_off'    : '<Alt>o',
-            'broadcast_group'  : '<Alt>g',
-            'broadcast_all'    : '<Alt>a',
+            'broadcast_off'    : '',
+            'broadcast_group'  : '',
+            'broadcast_all'    : '',
             'insert_number'    : '<Super>1',
             'insert_padded'    : '<Super>0',
             'edit_window_title': '<Control><Alt>w',
@@ -193,6 +202,7 @@ DEFAULTS = {
             'layout_launcher'  : '<Alt>l',
             'next_profile'     : '',
             'previous_profile' : '', 
+            'preferences'      : '',
             'help'             : 'F1'
         },
         'profiles': {
@@ -223,6 +233,7 @@ DEFAULTS = {
                 'scroll_on_output'      : False,
                 'scrollback_lines'      : 500,
                 'scrollback_infinite'   : False,
+                'disable_mousewheel_zoom': False,
                 'exit_action'           : 'close',
                 'palette'               : '#2e3436:#cc0000:#4e9a06:#c4a000:\
 #3465a4:#75507b:#06989a:#d3d7cf:#555753:#ef2929:#8ae234:#fce94f:\
@@ -234,17 +245,20 @@ DEFAULTS = {
                 'custom_command'        : '',
                 'use_system_font'       : True,
                 'use_theme_colors'      : False,
+                'bold_is_bright'        : False,
+                'line_height'           : 1.0,
                 'encoding'              : 'UTF-8',
                 'active_encodings'      : ['UTF-8', 'ISO-8859-1'],
                 'focus_on_close'        : 'auto',
                 'force_no_bell'         : False,
                 'cycle_term_tab'        : True,
                 'copy_on_selection'     : False,
-                'rewrap_on_resize'      : True,
                 'split_to_group'        : False,
                 'autoclean_groups'      : True,
                 'http_proxy'            : '',
                 'ignore_hosts'          : ['localhost','127.0.0.0/8','*.local'],
+                'background_image'      : '',
+                'background_alpha'      : 0.0
             },
         },
         'layouts': {
@@ -298,7 +312,7 @@ class Config(object):
             profile = options.profile
         dbg('Config::set_profile: Changing profile to %s' % profile)
         self.profile = profile
-        if not self.base.profiles.has_key(profile):
+        if profile not in self.base.profiles:
             dbg('Config::set_profile: %s does not exist, creating' % profile)
             self.base.profiles[profile] = copy(DEFAULTS['profiles']['default'])
 
@@ -313,7 +327,7 @@ class Config(object):
             # remove a profile
             err('Config::del_profile: Deleting in-use profile %s.' % profile)
             self.set_profile('default')
-        if self.base.profiles.has_key(profile):
+        if profile in self.base.profiles:
             del(self.base.profiles[profile])
         options = self.options_get()
         if options and options.profile == profile:
@@ -322,7 +336,7 @@ class Config(object):
 
     def rename_profile(self, profile, newname):
         """Rename a profile"""
-        if self.base.profiles.has_key(profile):
+        if profile in self.base.profiles:
             self.base.profiles[newname] = self.base.profiles[profile]
             del(self.base.profiles[profile])
             if profile == self.profile:
@@ -330,7 +344,7 @@ class Config(object):
 
     def list_profiles(self):
         """List all configured profiles"""
-        return(self.base.profiles.keys())
+        return(list(self.base.profiles.keys()))
 
     def add_layout(self, name, layout):
         """Add a new layout"""
@@ -342,18 +356,18 @@ class Config(object):
 
     def del_layout(self, layout):
         """Delete a layout"""
-        if self.base.layouts.has_key(layout):
+        if layout in self.base.layouts:
             del(self.base.layouts[layout])
 
     def rename_layout(self, layout, newname):
         """Rename a layout"""
-        if self.base.layouts.has_key(layout):
+        if layout in self.base.layouts:
             self.base.layouts[newname] = self.base.layouts[layout]
             del(self.base.layouts[layout])
 
     def list_layouts(self):
         """List all configured layouts"""
-        return(self.base.layouts.keys())
+        return(list(self.base.layouts.keys()))
 
     def connect_gsetting_callbacks(self):
         """Get system settings and create callbacks for changes"""
@@ -416,7 +430,7 @@ class Config(object):
         self.system_mono_font = None
         # Need to trigger a reconfigure to change active terminals immediately
         if "Terminator" not in globals():
-            from terminator import Terminator
+            from .terminator import Terminator
         Terminator().reconfigure()
 
     def save(self):
@@ -490,7 +504,7 @@ class ConfigBase(Borg):
         Borg.__init__(self, self.__class__.__name__)
 
         self.prepare_attributes()
-        import optionparse
+        from . import optionparse
         self.command_line_options = optionparse.options
         self.load()
 
@@ -585,7 +599,7 @@ class ConfigBase(Borg):
 
         configspec = ConfigObj(configspecdata)
         if DEBUG == True:
-            configspec.write(open('/tmp/terminator_configspec_debug.txt', 'w'))
+            configspec.write(open('/tmp/terminator_configspec_debug.txt', 'wb'))
         return(configspec)
 
     def load(self):
@@ -594,17 +608,16 @@ class ConfigBase(Borg):
             dbg('ConfigBase::load: config already loaded')
             return
 
-        if self.command_line_options:
-            if not self.command_line_options.config:
-                self.command_line_options.config = os.path.join(get_config_dir(), 'config')
+        if self.command_line_options and self.command_line_options.config:
             filename = self.command_line_options.config
         else:
             filename = os.path.join(get_config_dir(), 'config')
-
+            if not os.path.exists(filename):
+                filename = os.path.join(get_system_config_dir(), 'config')
         dbg('looking for config file: %s' % filename)
         try:
             configfile = open(filename, 'r')
-        except Exception, ex:
+        except Exception as ex:
             if not self.whined:
                 err('ConfigBase::load: Unable to open %s (%s)' % (filename, ex))
                 self.whined = True
@@ -617,7 +630,7 @@ class ConfigBase(Borg):
             parser = ConfigObj(configfile, configspec=configspec)
             validator = Validator()
             result = parser.validate(validator, preserve_errors=True)
-        except Exception, ex:
+        except Exception as ex:
             err('Unable to load configuration: %s' % ex)
             return
 
@@ -637,12 +650,12 @@ class ConfigBase(Borg):
             if section_name == 'profiles':
                 for profile in parser[section_name]:
                     dbg('ConfigBase::load: Processing profile: %s' % profile)
-                    if not section.has_key(section_name):
+                    if section_name not in section:
                         # FIXME: Should this be outside the loop?
                         section[profile] = copy(DEFAULTS['profiles']['default'])
                     section[profile].update(parser[section_name][profile])
             elif section_name == 'plugins':
-                if not parser.has_key(section_name):
+                if section_name not in parser:
                     continue
                 for part in parser[section_name]:
                     dbg('ConfigBase::load: Processing %s: %s' % (section_name,
@@ -657,7 +670,7 @@ class ConfigBase(Borg):
                            continue
                     section[layout] = parser[section_name][layout]
             elif section_name == 'keybindings':
-                if not parser.has_key(section_name):
+                if section_name not in parser:
                     continue
                 for part in parser[section_name]:
                     dbg('ConfigBase::load: Processing %s: %s' % (section_name,
@@ -669,7 +682,7 @@ class ConfigBase(Borg):
             else:
                 try:
                     section.update(parser[section_name])
-                except KeyError, ex:
+                except KeyError as ex:
                     dbg('ConfigBase::load: skipping missing section %s' %
                             section_name)
 
@@ -679,11 +692,11 @@ class ConfigBase(Borg):
         """Force a reload of the base config"""
         self.loaded = False
         self.load()
-        
+
     def save(self):
         """Save the config to a file"""
         dbg('ConfigBase::save: saving config')
-        parser = ConfigObj()
+        parser = ConfigObj(encoding='utf-8')
         parser.indent_type = '  '
 
         for section_name in ['global_config', 'keybindings']:
@@ -691,14 +704,20 @@ class ConfigBase(Borg):
             section = getattr(self, section_name)
             parser[section_name] = dict_diff(DEFAULTS[section_name], section)
 
+        from .configjson import JSON_PROFILE_NAME, JSON_LAYOUT_NAME
+
         parser['profiles'] = {}
         for profile in self.profiles:
+            if profile == JSON_PROFILE_NAME:
+                continue
             dbg('ConfigBase::save: Processing profile: %s' % profile)
             parser['profiles'][profile] = dict_diff(
                     DEFAULTS['profiles']['default'], self.profiles[profile])
 
         parser['layouts'] = {}
         for layout in self.layouts:
+            if layout == JSON_LAYOUT_NAME:
+                continue
             dbg('ConfigBase::save: Processing layout: %s' % layout)
             parser['layouts'][layout] = self.layouts[layout]
 
@@ -710,22 +729,37 @@ class ConfigBase(Borg):
         config_dir = get_config_dir()
         if not os.path.isdir(config_dir):
             os.makedirs(config_dir)
+
         try:
-            parser.write(open(self.command_line_options.config, 'w'))
-        except Exception, ex:
+            if self.command_line_options.config:
+                filename = self.command_line_options.config
+            else: 
+                filename = os.path.join(config_dir,'config')
+
+            if not os.path.isfile(filename):
+                open(filename, 'a').close()
+
+            backup_file = filename + '~'
+            shutil.copy2(filename, backup_file)
+
+            with open(filename, 'wb') as fh:
+                parser.write(fh)
+
+            os.remove(backup_file)
+        except Exception as ex:
             err('ConfigBase::save: Unable to save config: %s' % ex)
 
     def get_item(self, key, profile='default', plugin=None, default=None):
         """Look up a configuration item"""
-        if not self.profiles.has_key(profile):
+        if profile not in self.profiles:
             # Hitting this generally implies a bug
             profile = 'default'
 
-        if self.global_config.has_key(key):
+        if key in self.global_config:
             dbg('ConfigBase::get_item: %s found in globals: %s' %
                     (key, self.global_config[key]))
             return(self.global_config[key])
-        elif self.profiles[profile].has_key(key):
+        elif key in self.profiles[profile]:
             dbg('ConfigBase::get_item: %s found in profile %s: %s' % (
                     key, profile, self.profiles[profile][key]))
             return(self.profiles[profile][key])
@@ -745,14 +779,14 @@ class ConfigBase(Borg):
         dbg('ConfigBase::set_item: Setting %s=%s (profile=%s, plugin=%s)' %
                 (key, value, profile, plugin))
 
-        if self.global_config.has_key(key):
+        if key in self.global_config:
             self.global_config[key] = value
-        elif self.profiles[profile].has_key(key):
+        elif key in self.profiles[profile]:
             self.profiles[profile][key] = value
         elif key == 'keybindings':
             self.keybindings = value
         elif plugin is not None:
-            if not self.plugins.has_key(plugin):
+            if plugin not in self.plugins:
                 self.plugins[plugin] = {}
             self.plugins[plugin][key] = value
         else:
@@ -762,7 +796,7 @@ class ConfigBase(Borg):
 
     def get_plugin(self, plugin):
         """Return a whole tree for a plugin"""
-        if self.plugins.has_key(plugin):
+        if plugin in self.plugins:
             return(self.plugins[plugin])
 
     def set_plugin(self, plugin, tree):
@@ -797,7 +831,7 @@ class ConfigBase(Borg):
 
     def get_layout(self, layout):
         """Return a layout"""
-        if self.layouts.has_key(layout):
+        if layout in self.layouts:
             return(self.layouts[layout])
         else:
             err('layout does not exist: %s' % layout)
