@@ -1,4 +1,3 @@
-#!/usr/bin/python
 # Terminator by Chris Jones <cmsj@tenshu.net>
 # GPL v2 only
 """custom_commands.py - Terminator Plugin to add custom command menu entries"""
@@ -16,7 +15,7 @@ from terminatorlib.config import Config
 from terminatorlib.translation import _
 from terminatorlib.util import get_config_dir, err, dbg, gerr
 
-(CC_COL_ENABLED, CC_COL_NAME, CC_COL_COMMAND) = range(0,3)
+(CC_COL_ENABLED, CC_COL_NAME, CC_COL_COMMAND) = list(range(0,3))
 
 # Every plugin you want Terminator to load *must* be listed in 'AVAILABLE'
 AVAILABLE = ['CustomCommandsMenu']
@@ -35,13 +34,13 @@ class CustomCommandsMenu(plugin.MenuItem):
       noord_cmds = []
       for part in sections:
         s = sections[part]
-        if not (s.has_key("name") and s.has_key("command")):
-          print "CustomCommandsMenu: Ignoring section %s" % s
+        if not ("name" in s and "command" in s):
+          print("CustomCommandsMenu: Ignoring section %s" % s)
           continue
         name = s["name"]
         command = s["command"]
         enabled = s["enabled"] and s["enabled"] or False
-        if s.has_key("position"):
+        if "position" in s:
           self.cmd_list[int(s["position"])] = {'enabled' : enabled,
                                                'name' : name,
                                                'command' : command
@@ -58,6 +57,7 @@ class CustomCommandsMenu(plugin.MenuItem):
 
     def callback(self, menuitems, menu, terminal):
         """Add our menu items to the menu"""
+        submenus = {}
         item = Gtk.MenuItem.new_with_mnemonic(_('_Custom Commands'))
         menuitems.append(item)
 
@@ -77,16 +77,30 @@ class CustomCommandsMenu(plugin.MenuItem):
             continue
           exe = command['command'].split(' ')[0]
           iconinfo = theme.choose_icon([exe], Gtk.IconSize.MENU, Gtk.IconLookupFlags.USE_BUILTIN)
+          leaf_name = command['name'].split('/')[-1]
+          branch_names = command['name'].split('/')[:-1]
+          target_submenu = submenu
+          parent_submenu = submenu
+          for idx in range(len(branch_names)):
+            lookup_name = '/'.join(branch_names[0:idx+1])
+            target_submenu = submenus.get(lookup_name, None)
+            if not target_submenu:
+              item = Gtk.MenuItem(_(branch_names[idx]))
+              parent_submenu.append(item)
+              target_submenu = Gtk.Menu()
+              item.set_submenu(target_submenu)
+              submenus[lookup_name] = target_submenu
+            parent_submenu = target_submenu
           if iconinfo:
             image = Gtk.Image()
             image.set_from_icon_name(exe, Gtk.IconSize.MENU)
-            menuitem = Gtk.ImageMenuItem(command['name'])
+            menuitem = Gtk.ImageMenuItem(leaf_name)
             menuitem.set_image(image)
           else:
-            menuitem = Gtk.MenuItem(command["name"])
+            menuitem = Gtk.MenuItem(leaf_name)
           terminals = terminal.terminator.get_target_terms(terminal)
           menuitem.connect("activate", self._execute, {'terminals' : terminals, 'command' : command['command'] })
-          submenu.append(menuitem)
+          target_submenu.append(menuitem)
         
     def _save_config(self):
       config = Config()
@@ -112,7 +126,7 @@ class CustomCommandsMenu(plugin.MenuItem):
       if command[-1] != '\n':
         command = command + '\n'
       for terminal in data['terminals']:
-        terminal.vte.feed_child(command,  len(command))
+        terminal.vte.feed_child(command.encode())
 
     def configure(self, widget, data = None):
       ui = {}
@@ -125,6 +139,7 @@ class CustomCommandsMenu(plugin.MenuItem):
                         _("_OK"), Gtk.ResponseType.ACCEPT
                       )
                     )
+      dbox.set_transient_for(widget.get_toplevel())
 
       icon_theme = Gtk.IconTheme.get_default()
       if icon_theme.lookup_icon('terminator-custom-commands', 48, 0):
@@ -214,11 +229,13 @@ class CustomCommandsMenu(plugin.MenuItem):
 
 
       hbox.pack_start(button_box, False, True, 0)
+      self.dbox = dbox
       dbox.show_all()
       res = dbox.run()
       if res == Gtk.ResponseType.ACCEPT:
         self.update_cmd_list(store)
         self._save_config()
+      del(self.dbox)
       dbox.destroy()
       return
 
@@ -271,6 +288,7 @@ class CustomCommandsMenu(plugin.MenuItem):
                           _("_OK"), Gtk.ResponseType.ACCEPT
                         )
                       )
+      dialog.set_transient_for(self.dbox)
       table = Gtk.Table(3, 2)
 
       label = Gtk.Label(label=_("Enabled:"))
