@@ -1,6 +1,5 @@
-#!/usr/bin/env python2
-#    Terminator.util - misc utility functions
-#    Copyright (C) 2006-2010  cmsj@tenshu.net
+# Terminator.util - misc utility functions
+# Copyright (C) 2006-2010  cmsj@tenshu.net
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -16,6 +15,8 @@
 #  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 """Terminator.util - misc utility functions"""
 
+from __future__ import print_function
+
 import sys
 import cairo
 import os
@@ -25,6 +26,7 @@ import uuid
 import subprocess
 import gi
 
+
 try:
     gi.require_version('Gtk','3.0')
     from gi.repository import Gtk, Gdk
@@ -64,14 +66,14 @@ def dbg(log = ""):
         if DEBUGMETHODS != [] and method not in DEBUGMETHODS:
             return
         try:
-            print >> sys.stderr, "%s::%s: %s%s" % (classname, method, log, extra)
+            print("%s::%s: %s%s" % (classname, method, log, extra), file=sys.stderr)
         except IOError:
             pass
 
 def err(log = ""):
     """Print an error message"""
     try:
-        print >> sys.stderr, log
+        print(log, file=sys.stderr)
     except IOError:
         pass
 
@@ -95,7 +97,7 @@ def has_ancestor(widget, wtype):
 def manual_lookup():
     '''Choose the manual to open based on LANGUAGE'''
     available_languages = ['en']
-    base_url = 'http://terminator-gtk3.readthedocs.io/%s/latest/'
+    base_url = 'http://gnome-terminator.readthedocs.io/%s/latest/'
     target = 'en'   # default to English
     if 'LANGUAGE' in os.environ:
         languages = os.environ['LANGUAGE'].split(':')
@@ -183,6 +185,15 @@ def widget_pixbuf(widget, maxsize=None):
     
     return(scaledpixbuf)
 
+def get_system_config_dir():
+    system_config_dir = '/etc/xdg'
+    if 'XDG_CONFIG_DIRS' in os.environ.keys():
+        for sysconfdir in os.environ['XDG_CONFIG_DIRS'].split(":"):
+                if os.path.isdir(sysconfdir):
+                    system_config_dir = sysconfdir
+                    break
+    return(os.path.join(system_config_dir,'terminator'))
+
 def get_config_dir():
     """Expand all the messy nonsense for finding where ~/.config/terminator
     really is"""
@@ -279,7 +290,7 @@ def enumerate_descendants(parent):
     terminals"""
     # FIXME: Does having to import this here mean we should move this function
     # back to Container?
-    from factory import Factory
+    from .factory import Factory
 
     containerstmp = []
     containers = []
